@@ -1,16 +1,13 @@
 """This module defines the Ensembl ETL methods."""
 from .base import Base
 from gene import PROJECT_ROOT
-from gene.schemas import SourceName, NamespacePrefix, Strand, Gene, Meta
+from gene.schemas import SourceName, NamespacePrefix, Strand, Gene, SourceMeta
 import logging
 from gene.database import Database
 import gffutils
-import gzip
 from biocommons.seqrepo import SeqRepo
 from gene.vrs_locations import SequenceLocation
-from ftplib import FTP
-import shutil
-from os import remove
+
 
 logger = logging.getLogger('gene')
 logger.setLevel(logging.DEBUG)
@@ -21,16 +18,23 @@ class Ensembl(Base):
 
     def __init__(self,
                  database: Database,
-                 data_url='ftp://ftp.ensembl.org/pub/',
+                 host='ftp.ensembl.org',
+                 data_dir='pub/',
+                 fn='Homo_sapiens.GRCh38.102.gff3.gz'
                  ):
         """Initialize Ensembl ETL class.
 
         :param Database database: DynamoDB database
-        :param str data_url: URL to Ensembl's FTP site
+        :param str host: FTP host name
+        :param str data_dir: FTP data directory to use
+        :param str fn: Data file to download
         """
         self._database = database
         self._sequence_location = SequenceLocation()
-        self._data_url = data_url
+        self._data_url = f"ftp://{host}/{data_dir}{fn}"
+        self._host = host
+        self._data_dir = data_dir
+        self._fn = fn
         self._data_file_url = None
         self._version = '102'
         self._assembly = 'GRCh38'
@@ -38,21 +42,15 @@ class Ensembl(Base):
 
     def _download_data(self):
         """Download Ensembl GFF3 data file."""
-        logger.info('Downloading Ensembl...')
+        logger.info('Downloading Ensembl data file...')
         ens_dir = PROJECT_ROOT / 'data' / 'ensembl'
         ens_dir.mkdir(exist_ok=True, parents=True)
-        with FTP('ftp.ensembl.org') as ftp:
-            ftp.login()
-            ftp.cwd('pub/release-102/gff3/homo_sapiens/')
-            fn = 'ensembl_102.gff3'
-            gz_filepath = ens_dir / f'{fn}.gz'
-            with open(gz_filepath, 'wb') as fp:
-                ftp.retrbinary('RETR Homo_sapiens.GRCh38.102.gff3.gz',
-                               fp.write)
-        with gzip.open(gz_filepath, 'rb') as f_in:
-            with open(ens_dir / fn, 'wb') as f_out:
-                shutil.copyfileobj(f_in, f_out)
-        remove(gz_filepath)
+        self._ftp_download(self._host,
+                           f'{self._data_dir}release-102/gff3/homo_sapiens/',
+                           'ensembl_102.gff3',
+                           ens_dir,
+                           self._fn)
+        logger.info('Successfully downloaded Ensembl data file.')
 
     def _extract_data(self, *args, **kwargs):
         """Extract data from the Ensembl source."""
@@ -64,15 +62,16 @@ class Ensembl(Base):
 
     def _transform_data(self, *args, **kwargs):
         """Transform the Ensembl source."""
+        logger.info('Transforming Ensembl...')
         db = gffutils.create_db(str(self._data_src),
                                 dbfn=":memory:",
                                 force=True,
                                 merge_strategy="create_unique",
                                 keep_order=True)
 
-        seqrepo_dir = PROJECT_ROOT / 'data' / 'seqrepo' / '2020-11-27'
+        seqrepo_dir = PROJECT_ROOT / 'data' / 'seqrepo' / 'latest'
         if not seqrepo_dir.exists():
-            logger.error("Could not find seqrepo 2020-11-27 directory.")
+            logger.error("Could not find gene/data/seqrepo/latest directory.")
             raise NotADirectoryError("Could not find seqrepo "
                                      "2020-11-27 directory.")
         sr = SeqRepo(seqrepo_dir)
@@ -92,37 +91,8 @@ class Ensembl(Base):
                         gene = self._add_gene(f, sr, accession_numbers)
                         if gene:
                             assert Gene(**gene)
-                            self._load_symbol(gene, batch)
-                            self._load_other_ids(gene, batch)
-                            batch.put_item(Item=gene)
-
-    def _load_symbol(self, gene, batch):
-        """Load symbol records into database.
-
-        :param dict gene: A transformed gene record
-        :param BatchWriter batch: Object to write data to DynamoDB
-        """
-        symbol = {
-            'label_and_type': f"{gene['symbol'].lower()}##symbol",
-            'concept_id': f"{gene['concept_id'].lower()}",
-            'src_name': SourceName.ENSEMBL.value
-        }
-        batch.put_item(Item=symbol)
-
-    def _load_other_ids(self, gene, batch):
-        """Load other identifier records into database.
-
-        :param dict gene: A transformed gene record
-        :param BatchWriter batch: Object to write data to DynamoDB
-        """
-        if 'other_identifiers' in gene and gene['other_identifiers']:
-            for other_id in gene['other_identifiers']:
-                other_id = {
-                    'label_and_type': f"{other_id.lower()}##other_id",
-                    'concept_id': f"{gene['concept_id'].lower()}",
-                    'src_name': SourceName.ENSEMBL.value
-                }
-                batch.put_item(Item=other_id)
+                            self._load_gene(gene, batch)
+        logger.info('Successfully transformed Ensembl.')
 
     def _add_gene(self, f, sr, accession_numbers):
         """Create a transformed gene record.
@@ -148,11 +118,12 @@ class Ensembl(Base):
 
         gene['label_and_type'] = \
             f"{gene['concept_id'].lower()}##identity"
+        gene['item_type'] = 'identity'
 
         return gene
 
     def _add_attributes(self, f, gene):
-        """Add concept_id, symbol, and other_identifiers to a gene record.
+        """Add concept_id, symbol, xrefs, and associated_with to a gene record.
 
         :param gffutils.feature.Feature f: A gene from the data
         :param gene: A transformed gene record
@@ -160,7 +131,7 @@ class Ensembl(Base):
         attributes = {
             'ID': 'concept_id',
             'Name': 'symbol',
-            'description': 'other_identifiers'
+            'description': 'xrefs'
         }
 
         for attribute in f.attributes.items():
@@ -184,12 +155,11 @@ class Ensembl(Base):
                         src_id = val.split('Acc:')[-1].split(']')[0]
                         if ':' in src_id:
                             src_id = src_id.split(':')[-1]
-                        source = self._get_other_id_xref(src_name, src_id)
-                        if 'other_identifiers' in source:
-                            gene['other_identifiers'] = \
-                                source['other_identifiers']
-                        elif 'xrefs' in source:
+                        source = self._get_xref_associated_with(src_name, src_id)  # noqa: E501
+                        if 'xrefs' in source:
                             gene['xrefs'] = source['xrefs']
+                        elif 'associated_with' in source:
+                            gene['associated_with'] = source['associated_with']
                     continue
 
                 gene[attributes[key]] = val
@@ -207,8 +177,8 @@ class Ensembl(Base):
         return self._sequence_location.add_location(accession_numbers[f.seqid],
                                                     f, gene, sr)
 
-    def _get_other_id_xref(self, src_name, src_id):
-        """Get other identifier or xref.
+    def _get_xref_associated_with(self, src_name, src_id):
+        """Get xref or associated_with concept.
 
         :param str src_name: Source name
         :param src_id: The source's accession number
@@ -216,17 +186,17 @@ class Ensembl(Base):
         """
         source = dict()
         if src_name.startswith('HGNC'):
-            source['other_identifiers'] = \
+            source['xrefs'] = \
                 [f"{NamespacePrefix.HGNC.value}:{src_id}"]
         elif src_name.startswith('NCBI'):
-            source['other_identifiers'] = \
+            source['xrefs'] = \
                 [f"{NamespacePrefix.NCBI.value}:{src_id}"]
         elif src_name.startswith('UniProt'):
-            source['xrefs'] = [f"{NamespacePrefix.UNIPROT.value}:{src_id}"]
+            source['associated_with'] = [f"{NamespacePrefix.UNIPROT.value}:{src_id}"]  # noqa: E501
         elif src_name.startswith('miRBase'):
-            source['xrefs'] = [f"{NamespacePrefix.MIRBASE.value}:{src_id}"]
+            source['associated_with'] = [f"{NamespacePrefix.MIRBASE.value}:{src_id}"]  # noqa: E501
         elif src_name.startswith('RFAM'):
-            source['xrefs'] = [f"{NamespacePrefix.RFAM.value}:{src_id}"]
+            source['associated_with'] = [f"{NamespacePrefix.RFAM.value}:{src_id}"]  # noqa: E501
         return source
 
     def _load_data(self, *args, **kwargs):
@@ -238,7 +208,7 @@ class Ensembl(Base):
 
     def _add_meta(self, *args, **kwargs):
         """Add Ensembl metadata."""
-        metadata = Meta(
+        metadata = SourceMeta(
             data_license='custom',
             data_license_url='https://useast.ensembl.org/info/about'
                              '/legal/disclaimer.html',
@@ -265,3 +235,5 @@ class Ensembl(Base):
                 'genome_assemblies': metadata.genome_assemblies
             }
         )
+
+        self._load_meta(self._database, metadata, SourceName.ENSEMBL.value)
