@@ -1,14 +1,12 @@
 """This module defines the HGNC ETL methods."""
-from .base import Base, NORMALIZER_SRC_PREFIXES
-from gene import PROJECT_ROOT, DownloadException
+from .base import Base
+from gene import PROJECT_ROOT, PREFIX_LOOKUP
 from gene.schemas import SourceName, SymbolStatus, NamespacePrefix, Gene, \
-    Meta, Annotation, Chromosome
+    SourceMeta, Annotation, Chromosome
 from gene.database import Database
 import logging
 import json
-import requests
-from bs4 import BeautifulSoup
-import datetime
+import shutil
 import re
 from gene.vrs_locations import ChromosomeLocation
 
@@ -21,53 +19,38 @@ class HGNC(Base):
 
     def __init__(self,
                  database: Database,
-                 data_url='http://ftp.ebi.ac.uk/pub/databases/genenames/hgnc/',
-                 data_file_ext='json/hgnc_complete_set.json',
+                 host='ftp.ebi.ac.uk',
+                 data_dir='pub/databases/genenames/hgnc/json/',
+                 fn='hgnc_complete_set.json'
                  ):
         """Initialize HGNC ETL class.
 
         :param Database database: DynamoDB database
-        :param str data_url: URL to HGNC's FTP site
-        :param str data_file_ext: Extension to HGNC's current JSON data file
-                                  for the complete data set
+        :param str host: FTP host name
+        :param str data_dir: FTP data directory to use
+        :param str fn: Data file to download
         """
         self._database = database
         self._chromosome_location = ChromosomeLocation()
-        self._data_url = data_url
-        self._data_file_url = data_url + data_file_ext
+        self._data_url = f"ftp://{host}/{data_dir}{fn}"
+        self._host = host
+        self._data_dir = data_dir
+        self._fn = fn
         self._version = None
         self._load_data()
 
     def _download_data(self, *args, **kwargs):
         """Download HGNC JSON data file."""
-        logger.info('Downloading HGNC...')
-        response = requests.get(self._data_file_url, stream=True)
-        if response.status_code == 200:
-            r = requests.get(f"{self._data_url}/json/")
-        else:
-            logger.error(f"HGNC data file download failed with status code: "
-                         f"{response.status_code}")
-            raise DownloadException("HGNC data file download failed.")
-        if r.status_code == 200:
-            soup = BeautifulSoup(r.text, 'html.parser')
-            v_date = soup.find(
-                'a', text='hgnc_complete_set.json').next_sibling.split()[0]
-            self._version =\
-                datetime.datetime.strptime(v_date,
-                                           '%d-%b-%Y').strftime('%Y%m%d')
-
-            data_dir = PROJECT_ROOT / 'data' / 'hgnc'
-            data_dir.mkdir(exist_ok=True, parents=True)
-
-            with open(f"{PROJECT_ROOT}/data/hgnc/"
-                      f"hgnc_{self._version}.json", 'w+') as f:
-                f.write(json.dumps(response.json()))
-
-            logger.info('Finished downloading HGNC.')
-        else:
-            logger.error(f"HGNC download failed with status code: "
-                         f"{r.status_code}")
-            raise DownloadException("HGNC download failed.")
+        logger.info('Downloading HGNC data file...')
+        hgnc_data_dir = PROJECT_ROOT / 'data' / 'hgnc'
+        hgnc_data_dir.mkdir(exist_ok=True, parents=True)
+        tmp_fn = 'hgnc_version.json'
+        self._version = \
+            self._ftp_download(self._host, self._data_dir, tmp_fn,
+                               hgnc_data_dir, self._fn)
+        shutil.move(f"{hgnc_data_dir}/{tmp_fn}",
+                    f"{hgnc_data_dir}/hgnc_{self._version}.json")
+        logger.info('Successfully downloaded HGNC data file.')
 
     def _extract_data(self, *args, **kwargs):
         """Extract data from the HGNC source."""
@@ -79,6 +62,7 @@ class HGNC(Base):
 
     def _transform_data(self, *args, **kwargs):
         """Transform the HGNC source."""
+        logger.info('Transforming HGNC...')
         with open(self._data_src, 'r') as f:
             data = json.load(f)
 
@@ -90,6 +74,7 @@ class HGNC(Base):
                 gene['concept_id'] = r['hgnc_id'].lower()
                 gene['label_and_type'] = \
                     f"{gene['concept_id']}##identity"
+                gene['item_type'] = 'identity'
                 gene['symbol'] = r['symbol']
                 gene['label'] = r['name']
                 gene['src_name'] = SourceName.HGNC.value
@@ -102,43 +87,17 @@ class HGNC(Base):
                             SymbolStatus.WITHDRAWN.value
                 gene['src_name'] = SourceName.HGNC.value
 
-                # Store alias, other_identifier,
-                # prev_symbols, and location in gene record
+                # store alias, xref, associated_with, prev_symbols, location
                 self._get_aliases(r, gene)
-                self._get_other_ids_xrefs(r, gene)
+                self._get_xrefs_associated_with(r, gene)
                 if 'prev_symbol' in r:
                     self._get_previous_symbols(r, gene)
                 if 'location' in r:
                     self._get_location(r, gene)
 
                 assert Gene(**gene)
-                self._load_dynamodb(gene, batch)
-
-    def _load_dynamodb(self, gene, batch):
-        """Insert gene records into DynamoDB gene_concepts table
-
-        :param dict gene: A transformed gene record
-        :param BatchWriter batch: Object to write data to DynamoDB
-        """
-        self._load_approved_symbol(gene, batch)
-        self._load_aliases(gene, batch)
-        self._load_previous_symbols(gene, batch)
-        self._load_other_ids(gene, batch)
-        batch.put_item(Item=gene)
-
-    def _load_approved_symbol(self, gene, batch):
-        """Insert approved symbol data into the database.
-
-        :param dict gene: A transformed gene record
-        :param BatchWriter batch: Object to write data to DynamoDB
-        """
-        symbol = {
-            'label_and_type':
-                f"{gene['symbol'].lower()}##symbol",
-            'concept_id': f"{gene['concept_id'].lower()}",
-            'src_name': SourceName.HGNC.value
-        }
-        batch.put_item(Item=symbol)
+                self._load_gene(gene, batch)
+        logger.info('Successfully transformed HGNC.')
 
     def _get_aliases(self, r, gene):
         """Store aliases in a gene record.
@@ -157,23 +116,6 @@ class HGNC(Base):
         if alias_symbol or enzyme_id:
             gene['aliases'] = list(set(alias_symbol + enzyme_id))
 
-    def _load_aliases(self, gene, batch):
-        """Insert alias data into the database.
-
-        :param dict gene: A transformed gene record
-        :param BatchWriter batch: Object to write data to DynamoDB
-        """
-        if 'aliases' in gene:
-            aliases = {t.casefold(): t for t in gene['aliases']}
-
-            for alias in aliases:
-                alias = {
-                    'label_and_type': f"{alias}##alias",
-                    'concept_id': f"{gene['concept_id'].lower()}",
-                    'src_name': SourceName.HGNC.value
-                }
-                batch.put_item(Item=alias)
-
     def _get_previous_symbols(self, r, gene):
         """Store previous symbols in a gene record.
 
@@ -184,47 +126,14 @@ class HGNC(Base):
         if prev_symbols:
             gene['previous_symbols'] = list(set(prev_symbols))
 
-    def _load_previous_symbols(self, gene, batch):
-        """Load previous symbols to a gene record.
-
-        :param dict r: A gene record in the HGNC data file
-        :param dict gene: A transformed gene record
-        :param BatchWriter batch: Object to write data to DynamoDB
-        """
-        if 'previous_symbols' in gene:
-            prev_symbols = {t.casefold(): t for t in gene['previous_symbols']}
-
-            for prev_symbol in prev_symbols:
-                prev_symbol = {
-                    'label_and_type': f"{prev_symbol}##prev_symbol",
-                    'concept_id': f"{gene['concept_id'].lower()}",
-                    'src_name': SourceName.HGNC.value
-                }
-                batch.put_item(Item=prev_symbol)
-
-    def _load_other_ids(self, gene, batch):
-        """Insert other_id data into the database.
-
-        :param dict gene: A transformed gene record
-        :param BatchWriter batch: Object to write data to DynamoDB
-        """
-        if 'other_identifiers' in gene:
-            for other_id in gene['other_identifiers']:
-                other_id = {
-                    'label_and_type': f"{other_id.lower()}##other_id",
-                    'concept_id': f"{gene['concept_id'].lower()}",
-                    'src_name': SourceName.HGNC.value
-                }
-                batch.put_item(Item=other_id)
-
-    def _get_other_ids_xrefs(self, r, gene):
-        """Store other identifiers and/or xrefs in a gene record.
+    def _get_xrefs_associated_with(self, r, gene):
+        """Store xrefs and/or associated_with refs in a gene record.
 
         :param dict r: A gene record in the HGNC data file
         :param dict gene: A transformed gene record
         """
-        other_ids = list()
         xrefs = list()
+        associated_with = list()
         sources = [
             'entrez_id', 'ensembl_gene_id', 'vega_id', 'ucsc_id', 'ccds_id',
             'uniprot_ids', 'pubmed_id', 'cosmic', 'omim_id', 'mirbase',
@@ -243,32 +152,34 @@ class HGNC(Base):
                     key = src.split("_")[0]
                 else:
                     key = src
+
                 if key.upper() in NamespacePrefix.__members__:
-                    if NamespacePrefix[key.upper()]\
-                            .value in NORMALIZER_SRC_PREFIXES:
-                        self._get_other_id_xref(key, src, r, other_ids)
+                    if NamespacePrefix[key.upper()].value \
+                            in PREFIX_LOOKUP.keys():
+                        self._get_xref_associated_with(key, src, r, xrefs)
                     else:
-                        self._get_other_id_xref(key, src, r, xrefs)
+                        self._get_xref_associated_with(key, src, r,
+                                                       associated_with)
                 else:
                     logger.warning(f"{key} not in schemas.py")
 
-        if other_ids:
-            gene['other_identifiers'] = other_ids
         if xrefs:
             gene['xrefs'] = xrefs
+        if associated_with:
+            gene['associated_with'] = associated_with
 
-    def _get_other_id_xref(self, key, src, r, src_type):
-        """Add an other identifier or xref to a gene record.
+    def _get_xref_associated_with(self, key, src, r, src_type):
+        """Add an xref or associated_with ref to a gene record.
 
         :param str key: The source's name
         :param str src: HGNC's source field
         :param dict r: A gene record in the HGNC data file
-        :param list src_type: Either other identifiers list or xrefs list
+        :param list src_type: Either xrefs or associated_with list
         """
         if type(r[src]) == list:
-            for other_id in r[src]:
+            for xref in r[src]:
                 src_type.append(
-                    f"{NamespacePrefix[key.upper()].value}:{other_id}")
+                    f"{NamespacePrefix[key.upper()].value}:{xref}")
         else:
             if isinstance(r[src], str) and ':' in r[src]:
                 r[src] = r[src].split(':')[-1].strip()
@@ -367,10 +278,7 @@ class HGNC(Base):
 
     def _add_meta(self, *args, **kwargs):
         """Add HGNC metadata to the gene_metadata table."""
-        if self._data_url.startswith("http"):
-            self._data_url = f"ftp://{self._data_url.split('://')[-1]}"
-
-        metadata = Meta(
+        metadata = SourceMeta(
             data_license='custom',
             data_license_url='https://www.genenames.org/about/',
             version=self._version,
@@ -384,15 +292,4 @@ class HGNC(Base):
             genome_assemblies=[]
         )
 
-        self._database.metadata.put_item(
-            Item={
-                'src_name': SourceName.HGNC.value,
-                'data_license': metadata.data_license,
-                'data_license_url': metadata.data_license_url,
-                'version': metadata.version,
-                'data_url': metadata.data_url,
-                'rdp_url': metadata.rdp_url,
-                'data_license_attributes': metadata.data_license_attributes,
-                'genome_assemblies': metadata.genome_assemblies
-            }
-        )
+        self._load_meta(self._database, metadata, SourceName.HGNC.value)
