@@ -1,21 +1,18 @@
 """This module defines ETL methods for the NCBI data source."""
-from .base import Base, NORMALIZER_SRC_PREFIXES
-from gene import PROJECT_ROOT
+from .base import Base
+from gene import PROJECT_ROOT, PREFIX_LOOKUP
 from gene.database import Database
-from gene.schemas import Meta, Gene, SourceName, NamespacePrefix, \
-    Annotation, Chromosome
+from gene.schemas import SourceMeta, Gene, SourceName, NamespacePrefix, \
+    Annotation, Chromosome, SymbolStatus
 import logging
 from pathlib import Path
 import csv
 from datetime import datetime
-import gzip
-import shutil
-from os import remove
 import re
 import gffutils
 from biocommons.seqrepo import SeqRepo
 from gene.vrs_locations import SequenceLocation, ChromosomeLocation
-from ftplib import FTP
+
 
 logger = logging.getLogger('gene')
 logger.setLevel(logging.DEBUG)
@@ -26,18 +23,22 @@ class NCBI(Base):
 
     def __init__(self,
                  database: Database,
-                 data_url: str = 'ftp://ftp.ncbi.nlm.nih.gov/gene/DATA/',
+                 host='ftp.ncbi.nlm.nih.gov',
+                 data_dir='gene/DATA/',
                  assembly: str = 'GRCh38.p13'):
         """Construct the NCBI ETL instance.
 
         :param Database database: gene database for adding new data
-        :param str data_url: NCBI's FTP site containing gene source material
+        :param str host: FTP host name
+        :param str data_dir: FTP data directory to use
         :param str assembly: The genome assembly
         """
         self._database = database
         self._sequence_location = SequenceLocation()
         self._chromosome_location = ChromosomeLocation()
-        self._data_url = data_url
+        self._data_url = f"ftp://{host}"
+        self._host = host
+        self._data_dir = data_dir
         self._assembly = assembly
         self._extract_data()
         self._transform_data()
@@ -47,40 +48,31 @@ class NCBI(Base):
 
         :param str ncbi_dir: The NCBI data directory
         """
-        host = 'ftp.ncbi.nlm.nih.gov'
         version = datetime.today().strftime('%Y%m%d')
 
         # Download info
-        data_dir = 'gene/DATA/GENE_INFO/Mammalia/'
+        data_dir = f'{self._data_dir}GENE_INFO/Mammalia/'
         fn = f'ncbi_info_{version}.tsv'
         data_fn = 'Homo_sapiens.gene_info.gz'
-        self._ftp_download(host, data_dir, fn, ncbi_dir, data_fn)
+        logger.info('Downloading NCBI gene_info....')
+        self._ftp_download(self._host, data_dir, fn, ncbi_dir, data_fn)
+        logger.info('Successfully downloaded NCBI gene_info.')
 
         # Download history
-        data_dir = 'gene/DATA/'
         fn = f'ncbi_history_{version}.tsv'
         data_fn = 'gene_history.gz'
-        self._ftp_download(host, data_dir, fn, ncbi_dir, data_fn)
+        logger.info('Downloading NCBI gene_history...')
+        self._ftp_download(self._host, self._data_dir, fn, ncbi_dir, data_fn)
+        logger.info('Successfully downloaded NCBI gene_history.')
 
         # Download gff
         data_dir = 'genomes/refseq/vertebrate_mammalian/Homo_sapiens/' \
                    'latest_assembly_versions/GCF_000001405.39_GRCh38.p13/'
         fn = f'ncbi_{self._assembly}.gff'
         data_fn = 'GCF_000001405.39_GRCh38.p13_genomic.gff.gz'
-        self._ftp_download(host, data_dir, fn, ncbi_dir, data_fn)
-
-    def _ftp_download(self, host: str, data_dir: str, fn: str, ncbi_dir: Path,
-                      data_fn: str):
-        with FTP(host) as ftp:
-            ftp.login()
-            ftp.cwd(data_dir)
-            gz_filepath = ncbi_dir / f'{fn}.gz'
-            with open(gz_filepath, 'wb') as fp:
-                ftp.retrbinary(f'RETR {data_fn}', fp.write)
-            with gzip.open(gz_filepath, 'rb') as f_in:
-                with open(ncbi_dir / fn, 'wb') as f_out:
-                    shutil.copyfileobj(f_in, f_out)
-            remove(gz_filepath)
+        logger.info('Downloading NCBI gff data file...')
+        self._ftp_download(self._host, data_dir, fn, ncbi_dir, data_fn)
+        logger.info('Successfully downloaded NCBI gff data file.')
 
     def _files_downloaded(self, data_dir: Path) -> bool:
         """Check whether needed source files exist.
@@ -133,33 +125,46 @@ class NCBI(Base):
         history = csv.reader(history_file, delimiter='\t')
         next(history)
         prev_symbols = {}
-        for row in history:
-            # Only interested in rows that have homo sapiens tax id
-            if row[0] == '9606' and row[1] != '-':
-                gene_id = row[1]
-                if gene_id in prev_symbols.keys():
-                    prev_symbols[gene_id].append(row[3])
-                else:
-                    prev_symbols[gene_id] = [row[3]]
+        with self._database.genes.batch_writer() as batch:
+            for row in history:
+                # Only interested in rows that have homo sapiens tax id
+                if row[0] == '9606':
+                    if row[1] != '-':
+                        gene_id = row[1]
+                        if gene_id in prev_symbols.keys():
+                            prev_symbols[gene_id].append(row[3])
+                        else:
+                            prev_symbols[gene_id] = [row[3]]
+                    else:
+                        # Load discontinued genes
+                        params = {
+                            'concept_id':
+                                f'{NamespacePrefix.NCBI.value.lower()}:'
+                                f'{row[2]}',
+                            'symbol': row[3],
+                            'symbol_status': SymbolStatus.DISCONTINUED.value
+                        }
+                        assert Gene(**params)
+                        self._load_gene(params, batch)
         history_file.close()
         return prev_symbols
 
-    def _add_other_ids_xrefs(self, val, params):
-        """Add other identifiers and xrefs to a transformed gene.
+    def _add_xrefs_associated_with(self, val, params):
+        """Add xrefs and associated_with refs to a transformed gene.
 
         :param list val: A list of source ids for a given gene
         :param dict params: A transformed gene record
         """
         params['xrefs'] = []
-        params['other_identifiers'] = []
+        params['associated_with'] = []
         for src in val:
             src_name = src.split(':')[0].upper()
             src_id = src.split(':')[-1]
             if src_name == "GENEID":
                 params['concept_id'] = f"{NamespacePrefix.NCBI.value}:{src_id}"
             elif src_name in NamespacePrefix.__members__ and \
-                    NamespacePrefix[src_name].value in NORMALIZER_SRC_PREFIXES:
-                params['other_identifiers'].append(
+                    NamespacePrefix[src_name].value in PREFIX_LOOKUP:
+                params['xrefs'].append(
                     f"{NamespacePrefix[src_name].value}"
                     f":{src_id}")
             else:
@@ -172,13 +177,13 @@ class NCBI(Base):
                 else:
                     prefix = None
                 if prefix:
-                    params['xrefs'].append(f"{prefix}:{src_id}")
+                    params['associated_with'].append(f"{prefix}:{src_id}")
                 else:
                     logger.info(f"{src_name} is not in NameSpacePrefix.")
         if not params['xrefs']:
             del params['xrefs']
-        if not params['other_identifiers']:
-            del params['other_identifiers']
+        if not params['associated_with']:
+            del params['associated_with']
 
     def _get_gene_info(self, prev_symbols):
         """Store genes from NCBI info file.
@@ -202,10 +207,10 @@ class NCBI(Base):
                 params['aliases'] = row[4].split('|')
             else:
                 params['aliases'] = []
-            # get other identifiers
+            # get associated_with
             if row[5] != '-':
-                xrefs = row[5].split('|')
-                self._add_other_ids_xrefs(xrefs, params)
+                associated_with = row[5].split('|')
+                self._add_xrefs_associated_with(associated_with, params)
             # get chromosome location
             vrs_chr_location = self._get_vrs_chr_location(row, params)
             if 'exclude' in vrs_chr_location:
@@ -247,60 +252,6 @@ class NCBI(Base):
                         gene = self._add_gff_gene(db, f, sr, f_id)
                         info_genes[gene['symbol']] = gene
 
-    def _load_data(self, item, batch):
-        """Load individual Gene item.
-
-        :param dict item: A transformed gene record
-        :param batch: boto3 batch writer
-        """
-        concept_id_lower = item['concept_id'].lower()
-
-        pk = f"{item['symbol'].lower()}##symbol"
-        batch.put_item(Item={
-            'label_and_type': pk,
-            'concept_id': concept_id_lower,
-            'src_name': SourceName.NCBI.value
-        })
-
-        if 'aliases' in item and item['aliases']:
-            item['aliases'] = list(set(item['aliases']))
-            aliases = {alias.lower() for alias in item['aliases']}
-            for alias in aliases:
-                pk = f"{alias}##alias"
-                batch.put_item(Item={
-                    'label_and_type': pk,
-                    'concept_id': concept_id_lower,
-                    'src_name': SourceName.NCBI.value
-                })
-
-        if 'previous_symbols' in item and item['previous_symbols']:
-            item['previous_symbols'] = list(set(item['previous_symbols']))
-            item_prev_symbols = {s.lower() for s in item['previous_symbols']}
-            for symbol in item_prev_symbols:
-                pk = f"{symbol}##prev_symbol"
-                batch.put_item(Item={
-                    'label_and_type': pk,
-                    'concept_id': concept_id_lower,
-                    'src_name': SourceName.NCBI.value
-                })
-
-        if 'other_identifiers' in item and item['other_identifiers']:
-            for other_id in item['other_identifiers']:
-                pk = f"{other_id.lower()}##other_id"
-                batch.put_item(Item={
-                    'label_and_type': pk,
-                    'concept_id': concept_id_lower,
-                    'src_name': SourceName.NCBI.value
-                })
-
-        filtered_item = {k: v for k, v in item.items() if v is not None}
-        item.clear()
-        item.update(filtered_item)
-
-        item['label_and_type'] = f"{concept_id_lower}##identity"
-        item['src_name'] = SourceName.NCBI.value
-        batch.put_item(Item=item)
-
     def _add_gff_gene(self, db, f, sr, f_id):
         """Create a transformed gene recor from NCBI gff file.
 
@@ -324,7 +275,7 @@ class NCBI(Base):
         return params
 
     def _add_attributes(self, f, gene):
-        """Add concept_id, symbol, and other_identifiers to a gene record.
+        """Add concept_id, symbol, and xrefs/associated_with to a gene record.
 
         :param gffutils.feature.Feature f: A gene from the data
         :param gene: A transformed gene record
@@ -340,7 +291,7 @@ class NCBI(Base):
                     val = val[0]
 
                 if key == 'Dbxref':
-                    self._add_other_ids_xrefs(val, gene)
+                    self._add_xrefs_associated_with(val, gene)
                 elif key == 'Name':
                     gene['symbol'] = val
 
@@ -359,26 +310,26 @@ class NCBI(Base):
         return self._sequence_location.add_location(gene.seqid, gene,
                                                     params, sr)
 
-    def _get_other_id_xref(self, src_name, src_id):
-        """Get other identifier or xref.
+    def _get_xref_associated_with(self, src_name, src_id):
+        """Get xref or associated_with ref.
 
         :param str src_name: Source name
         :param src_id: The source's accession number
-        :return: A dict containing an other identifier or xref
+        :return: A dict containing an xref or associated_with ref
         """
         source = dict()
         if src_name.startswith('HGNC'):
-            source['other_identifiers'] = \
+            source['xrefs'] = \
                 [f"{NamespacePrefix.HGNC.value}:{src_id}"]
         elif src_name.startswith('NCBI'):
-            source['other_identifiers'] = \
+            source['xrefs'] = \
                 [f"{NamespacePrefix.NCBI.value}:{src_id}"]
         elif src_name.startswith('UniProt'):
-            source['xrefs'] = [f"{NamespacePrefix.UNIPROT.value}:{src_id}"]
+            source['associated_with'] = [f"{NamespacePrefix.UNIPROT.value}:{src_id}"]  # noqa E501
         elif src_name.startswith('miRBase'):
-            source['xrefs'] = [f"{NamespacePrefix.MIRBASE.value}:{src_id}"]
+            source['associated_with'] = [f"{NamespacePrefix.MIRBASE.value}:{src_id}"]  # noqa E501
         elif src_name.startswith('RFAM'):
-            source['xrefs'] = [f"{NamespacePrefix.RFAM.value}:{src_id}"]
+            source['associated_with'] = [f"{NamespacePrefix.RFAM.value}:{src_id}"]  # noqa E501
         return source
 
     def _get_vrs_chr_location(self, row, params):
@@ -553,11 +504,12 @@ class NCBI(Base):
 
     def _transform_data(self):
         """Modify data and pass to loading functions."""
+        logger.info('Transforming NCBI...')
         self._add_meta()
         prev_symbols = self._get_prev_symbols()
         info_genes = self._get_gene_info(prev_symbols)
 
-        seqrepo_dir = PROJECT_ROOT / 'data' / 'seqrepo' / '2020-11-27'
+        seqrepo_dir = PROJECT_ROOT / 'data' / 'seqrepo' / 'latest'
         sr = SeqRepo(seqrepo_dir)
 
         # create db for gff file
@@ -572,14 +524,12 @@ class NCBI(Base):
         with self._database.genes.batch_writer() as batch:
             for gene in info_genes.keys():
                 assert Gene(**info_genes[gene])
-                self._load_data(info_genes[gene], batch)
+                self._load_gene(info_genes[gene], batch)
+        logger.info('Successfully transformed NCBI.')
 
     def _add_meta(self):
         """Load metadata"""
-        if self._data_url.startswith("http"):
-            self._data_url = f"ftp://{self._data_url.split('://')[-1]}"
-
-        metadata = Meta(
+        metadata = SourceMeta(
             data_license="custom",
             data_license_url="https://www.ncbi.nlm.nih.gov/home/"
                              "about/policies/",
@@ -594,13 +544,4 @@ class NCBI(Base):
             genome_assemblies=[self._assembly]
         )
 
-        self._database.metadata.put_item(Item={
-            'src_name': SourceName.NCBI.value,
-            'data_license': metadata.data_license,
-            'data_license_url': metadata.data_license_url,
-            'version': metadata.version,
-            'data_url': metadata.data_url,
-            'rdp_url': metadata.rdp_url,
-            'data_license_attributes': metadata.data_license_attributes,
-            'genome_assemblies': metadata.genome_assemblies
-        })
+        self._load_meta(self._database, metadata, SourceName.NCBI.value)
