@@ -2,23 +2,13 @@
 import re
 from typing import List, Dict, Set
 from uvicorn.config import logger
+from .version import __version__
+from gene import NAMESPACE_LOOKUP, PREFIX_LOOKUP, ITEM_TYPES
 from gene.database import Database
-from gene.schemas import Gene, Meta, MatchType, SourceName, \
-    NamespacePrefix, SourceIDAfterNamespace
+from gene.schemas import Gene, SourceMeta, MatchType, SourceName, ServiceMeta
 from botocore.exceptions import ClientError
 from boto3.dynamodb.conditions import Key
-
-# use to fetch source name from schema based on concept id namespace
-# e.g. {'hgnc': 'HGNC'}
-PREFIX_LOOKUP = {v.value: SourceName[k].value
-                 for k, v in NamespacePrefix.__members__.items()
-                 if k in SourceName.__members__.keys()}
-
-# use to generate namespace prefix from source ID value
-# e.g. {'ENSG': 'ensembl'}
-NAMESPACE_LOOKUP = {v.value.lower(): NamespacePrefix[k].value
-                    for k, v in SourceIDAfterNamespace.__members__.items()
-                    if v.value != ''}
+from datetime import datetime
 
 
 class InvalidParameterException(Exception):
@@ -45,11 +35,11 @@ class QueryHandler:
         """
         self.db = Database(db_url=db_url, region_name=db_region)
 
-    def emit_warnings(self, query_str: str) -> Dict[str, str]:
+    def emit_warnings(self, query_str: str) -> List[str]:
         """Emit warnings if query contains non breaking space characters.
 
         :param str query_str: query string
-        :return: dict keying warning type to warning description
+        :return: List of warnings
         """
         warnings = []
         nbsp = re.search('\xa0|&nbsp;', query_str)
@@ -60,11 +50,11 @@ class QueryHandler:
             )
         return warnings
 
-    def fetch_meta(self, src_name: str) -> Meta:
+    def fetch_meta(self, src_name: str) -> SourceMeta:
         """Fetch metadata for src_name.
 
         :param str src_name: name of source to get metadata for
-        :return: Meta object containing source metadata
+        :return: SourceMeta object containing source metadata
         """
         if src_name in self.db.cached_sources.keys():
             return self.db.cached_sources[src_name]
@@ -72,11 +62,11 @@ class QueryHandler:
             try:
                 db_response = self.db.metadata.get_item(Key={'src_name':
                                                              src_name})
-                response = Meta(**db_response['Item'])
+                response = SourceMeta(**db_response['Item'])
                 self.db.cached_sources[src_name] = response
                 return response
             except ClientError as e:
-                print(e.response['Error']['Message'])
+                logger.error(e.response['Error']['Message'])
 
     def add_record(self,
                    response: Dict[str, Dict],
@@ -87,17 +77,11 @@ class QueryHandler:
         :param Dict[str, Dict] response: in-progress response object to return
             to client
         :param Dict item: Item retrieved from DynamoDB
-        :param MatchType match_type: type of query match
+        :param MatchTy pe match_type: type of query match
         :return: Tuple containing updated response object, and string
             containing name of the source of the match
         """
         del item['label_and_type']
-        attr_types = ['aliases', 'other_identifiers', 'previous_symbols',
-                      'xrefs', 'locations', 'location_annotations']
-        for attr_type in attr_types:
-            if attr_type not in item.keys():
-                item[attr_type] = []
-
         gene = Gene(**item)
         src_name = item['src_name']
 
@@ -108,7 +92,7 @@ class QueryHandler:
             matches[src_name] = {
                 'match_type': match_type,
                 'records': [gene],
-                'meta_': self.fetch_meta(src_name)
+                'source_meta_': self.fetch_meta(src_name)
             }
         elif matches[src_name]['match_type'].value == match_type.value:
             matches[src_name]['records'].append(gene)
@@ -141,7 +125,7 @@ class QueryHandler:
                 (response, src) = self.add_record(response, match, match_type)
                 matched_sources.add(src)
             except ClientError as e:
-                print(e.response['Error']['Message'])
+                logger.error(e.response['Error']['Message'])
 
         return (response, matched_sources)
 
@@ -157,7 +141,7 @@ class QueryHandler:
                 resp['source_matches'][src_name] = {
                     'match_type': MatchType.NO_MATCH,
                     'records': [],
-                    'meta_': self.fetch_meta(src_name)
+                    'source_meta_': self.fetch_meta(src_name)
                 }
         return resp
 
@@ -183,7 +167,7 @@ class QueryHandler:
                 if len(result['Items']) > 0:
                     concept_id_items += result['Items']
             except ClientError as e:
-                print(e.response['Error']['Message'])
+                logger.error(e.response['Error']['Message'])
         for prefix in [p for p in NAMESPACE_LOOKUP.keys() if
                        query.startswith(p)]:
             pk = f'{NAMESPACE_LOOKUP[prefix].lower()}:{query}##identity'
@@ -195,7 +179,7 @@ class QueryHandler:
                 if len(result['Items']) > 0:  # TODO remove check?
                     concept_id_items += result['Items']
             except ClientError as e:
-                print(e.response['Error']['Message'])
+                logger.error(e.response['Error']['Message'])
 
         for item in concept_id_items:
             (resp, src_name) = self.add_record(resp, item,
@@ -229,7 +213,7 @@ class QueryHandler:
                 )
                 sources = sources - matched_srcs
         except ClientError as e:
-            print(e.response['Error']['Message'])
+            logger.error(e.response['Error']['Message'])
         return (resp, sources)
 
     def response_keyed(self, query: str, sources: List[str]) -> Dict:
@@ -257,8 +241,7 @@ class QueryHandler:
         if len(sources) == 0:
             return resp
 
-        match_types = ['symbol', 'prev_symbol', 'alias', 'other_id']
-        for match in match_types:
+        for match in ITEM_TYPES.values():
             (resp, sources) = self.check_match_type(
                 query_l, resp, sources, match)
             if len(sources) == 0:
@@ -354,4 +337,9 @@ class QueryHandler:
         else:
             resp = self.response_list(query_str, query_sources)
 
+        resp['service_meta_'] = ServiceMeta(
+            version=__version__,
+            response_datetime=datetime.now(),
+        )
+
         return resp
