@@ -5,6 +5,7 @@ from typing import Type, List, Optional, Dict, Union, Any
 from pydantic import BaseModel, StrictBool
 from enum import Enum, IntEnum
 from pydantic.fields import Field
+from datetime import datetime
 
 
 class SymbolStatus(str, Enum):
@@ -12,6 +13,7 @@ class SymbolStatus(str, Enum):
 
     WITHDRAWN = "withdrawn"
     APPROVED = "approved"
+    DISCONTINUED = "discontinued"
 
 
 class Strand(str, Enum):
@@ -176,12 +178,12 @@ class Gene(BaseModel):
     symbol_status: Optional[SymbolStatus]
     label: Optional[str]
     strand: Optional[Strand]
-    location_annotations: Optional[List[str]]
-    locations: Optional[List[Union[ChromosomeLocation, SequenceLocation]]]
-    aliases: Optional[List[str]]
-    previous_symbols: Optional[List[str]]
-    other_identifiers: Optional[List[str]]
-    xrefs: Optional[List[str]]
+    location_annotations: Optional[List[str]] = []
+    locations: Optional[List[Union[ChromosomeLocation, SequenceLocation]]] = []
+    aliases: Optional[List[str]] = []
+    previous_symbols: Optional[List[str]] = []
+    xrefs: Optional[List[str]] = []
+    associated_with: Optional[List[str]] = []
 
     class Config:
         """Configure model"""
@@ -202,7 +204,7 @@ class Gene(BaseModel):
                 "symbol": "BRAF",
                 "previous_symbols": [],
                 "aliases": [],
-                "other_identifiers": [],
+                "xrefs": [],
                 "symbol_status": None,
                 "strand": "-",
                 "location": []
@@ -224,7 +226,8 @@ class MatchType(IntEnum):
     SYMBOL = 100
     PREV_SYMBOL = 80
     ALIAS = 60
-    OTHER_ID = 60
+    XREF = 60
+    ASSOCIATED_WITH = 60
     FUZZY_MATCH = 20
     NO_MATCH = 0
 
@@ -286,7 +289,18 @@ class DataLicenseAttributes(BaseModel):
     attribution: StrictBool
 
 
-class Meta(BaseModel):
+class ItemTypes(str, Enum):
+    """Item types used in DynamoDB."""
+
+    # Must be in descending MatchType order.
+    SYMBOL = 'symbol'
+    PREVIOUS_SYMBOLS = 'prev_symbol'
+    ALIASES = 'alias'
+    XREFS = 'xref'
+    ASSOCIATED_WITH = 'associated_with'
+
+
+class SourceMeta(BaseModel):
     """Metadata for a given source to return in response object."""
 
     data_license: str
@@ -302,7 +316,7 @@ class Meta(BaseModel):
 
         @staticmethod
         def schema_extra(schema: Dict[str, Any],
-                         model: Type['Meta']) -> None:
+                         model: Type['SourceMeta']) -> None:
             """Configure OpenAPI schema"""
             if 'title' in schema.keys():
                 schema.pop('title', None)
@@ -330,7 +344,7 @@ class MatchesKeyed(BaseModel):
 
     match_type: MatchType
     records: List[Gene]
-    meta_: Meta
+    source_meta_: SourceMeta
 
     class Config:
         """Enables orm_mode"""
@@ -347,7 +361,7 @@ class MatchesKeyed(BaseModel):
                 "NCBI": {
                     "match_type": 0,
                     "records": [],
-                    "meta_": {
+                    "source_meta_": {
                         "data_license": "custom",
                         "data_license_url": "https://www.ncbi.nlm.nih.gov/home/about/policies/",  # noqa: E501
                         "version": "20201215",
@@ -372,7 +386,7 @@ class MatchesListed(BaseModel):
     source: SourceName
     match_type: MatchType
     records: List[Gene]
-    meta_: Meta
+    source_meta_: SourceMeta
 
     class Config:
         """Enables orm_mode"""
@@ -389,7 +403,7 @@ class MatchesListed(BaseModel):
                 "source": "NCBI",
                 "match_type": 0,
                 "records": [],
-                "meta_": {
+                "source_meta_": {
                     "data_license": "custom",
                     "data_license_url": "https://www.ncbi.nlm.nih.gov/home/about/policies/",  # noqa: E501
                     "version": "20201215",
@@ -405,12 +419,40 @@ class MatchesListed(BaseModel):
             }
 
 
+class ServiceMeta(BaseModel):
+    """Metadata regarding the gene-normalization service."""
+
+    name = 'gene-normalizer'
+    version: str
+    response_datetime: datetime
+    url = 'https://github.com/cancervariants/gene-normalization'
+
+    class Config:
+        """Enables orm_mode"""
+
+        @staticmethod
+        def schema_extra(schema: Dict[str, Any],
+                         model: Type['ServiceMeta']) -> None:
+            """Configure OpenAPI schema"""
+            if 'title' in schema.keys():
+                schema.pop('title', None)
+            for prop in schema.get('properties', {}).values():
+                prop.pop('title', None)
+            schema['example'] = {
+                'name': 'gene-normalizer',
+                'version': '0.1.0',
+                'response_datetime': '2021-04-05T16:44:15.367831',
+                'url': 'https://github.com/cancervariants/gene-normalization'
+            }
+
+
 class Service(BaseModel):
     """Core response schema containing matches for each source"""
 
     query: str
     warnings: Optional[List]
     source_matches: Union[Dict[SourceName, MatchesKeyed], List[MatchesListed]]
+    service_meta_: ServiceMeta
 
     class Config:
         """Enables orm_mode"""
@@ -437,13 +479,13 @@ class Service(BaseModel):
                                 "symbol": "BRAF",
                                 "previous_symbols": [],
                                 "aliases": [],
-                                "other_identifiers": [],
+                                "xrefs": [],
                                 "symbol_status": None,
                                 "strand": "-",
                                 "locations": []
                             }
                         ],
-                        "meta_": {
+                        "source_meta_": {
                             "data_license": "custom",
                             "data_license_url": "https://uswest.ensembl.org/info/about/legal/index.html",  # noqa: E501
                             "version": "102",
@@ -457,5 +499,11 @@ class Service(BaseModel):
                             "genome_assemblies": "GRCh38"
                         }
                     }
-                ]
+                ],
+                "service_meta_": {
+                    'name': 'gene-normalizer',
+                    'version': '0.1.0',
+                    'response_datetime': '2021-04-05T16:44:15.367831',
+                    'url': 'https://github.com/cancervariants/gene-normalization'  # noqa: E501
+                }
             }
